/*
 * Part of BarGraph widget
 *
 * Created: 2018-11-22
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include "bargraph.h"
#include <QPicture>
#include <QPainter>
#include <QLabel>
#include <QPalette>
#include <QPen>
#include <QtMath>


BarGraph::BarGraph(QWidget *parent)
    : QWidget(parent)
{
    setAutoFillBackground(false);
    setAttribute(Qt::WA_NoSystemBackground, true);
    this->setContentsMargins(0,0,0,0);

    labelAxis = new QLabel(this);
    labelAxis->setObjectName(QStringLiteral("__qt__passive_label"));
    labelAxis->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    labelAxis->setGeometry(0, 0, this->width(), this->height());
    labelAxis->setAttribute(Qt::WA_TranslucentBackground);
    labelAxis->setLineWidth(0);
    labelAxis->setVisible(true);

    labelBarL = new QLabel(this);
    labelBarL->setObjectName(QStringLiteral("__qt__passive_label"));
    labelBarL->setLineWidth(0);

    labelBarR = new QLabel(this);
    labelBarR->setObjectName(QStringLiteral("__qt__passive_label"));
    labelBarR->setLineWidth(0);
    labelBarR->setVisible(true);

    this->axis.majorTickRising = true;
    this->axis.minorTick = 6;
    this->axis.majorTick = 10;
    this->axis.tickWidth = 2;
    this->axis.barColor = palette().base().color();
    this->axis.tickColor = palette().foreground().color();
    this->bar.minimum = 0;
    this->bar.maximum = 100;
    this->bar.value = 0;
    this->alarm.visible = false;
    this->alarm.critical = "#f6b0b0";
    this->alarm.warning = "#f6f278";
    this->alarm.topCritical = 90;
    this->alarm.topWarning = 80;
    this->alarm.bottomWarning = 20;
    this->alarm.bottomCritical = 10;
}

void BarGraph::resizeEvent(QResizeEvent *)
{
    labelAxis->setGeometry(0, 0, this->width(), this->height());
    drawAxis();
}

void BarGraph::drawTick(QPainter *p, int x, int length, Axis ax, int offset)
{
    int xpos;
    xpos = x * (ax.tickWidth + TICK_SPACE) + offset;
    int start = length;
    if (ax.majorTickRising) start = getBarWidth(ax);
    p->drawLine(xpos, start - length, xpos, start);
}

void BarGraph::drawAxis(Axis ax, int width)
{
    QPixmap pixmap(labelAxis->size());
    pixmap.fill(QColor("transparent"));
    QPainter painter(&pixmap);
    painter.setPen(QPen(ax.tickColor, ax.tickWidth, Qt::SolidLine, Qt::FlatCap));
    int tickStep = getTickStep(width, ax);
    int offset;
    offset = getOffset(width, ax);
    int tick = 0;
    for (int i = 0; i < 4; i++) {
        drawTick(&painter, tick++, ax.majorTick, ax, offset);
        for (int j = 0; j < tickStep; j++) {
            drawTick(&painter, tick++, ax.minorTick, ax, offset);
        }
    }
    drawTick(&painter, tick, ax.majorTick, ax, offset);
    drawBar(width, ax, this->bar);
    labelAxis->setPixmap(pixmap);
    this->setMinimumHeight(getBarWidth(this->axis));
}

void  BarGraph::drawAxis()
{
    drawAxis(this->axis, this->width());
}

QSize BarGraph::sizeHint() const
{
    return QSize(getAxisWidth(24, this->axis), getBarWidth(this->axis));
}

QSize BarGraph::minimumSizeHint() const
{
    return QSize(getAxisWidth(1, this->axis), getBarWidth(this->axis));
}

void BarGraph::setMinorTick(int length)
{
    this->axis.minorTick = length;
    drawAxis();
}

void BarGraph::setMajorTick(int length)
{
    this->axis.majorTick = length;
    drawAxis();
}

void BarGraph::setTickWidth(int width)
{
    if (width < 1) width = 1;
    this->axis.tickWidth = width;
    drawAxis();
}

void BarGraph::setMajorTickRising(bool rising)
{
    this->axis.majorTickRising = rising;
    drawAxis();
}

QString BarGraph::addTransparency(QString color)
{
    color.insert(1, BAR_TRANSPARENCY);
    return color;
}

int BarGraph::getBarWidth(Axis ax) const
{
    if (ax.minorTick > ax.majorTick) return ax.minorTick;
    return ax.majorTick;
}

int BarGraph::getTickStep(int width, Axis ax)
{
    int result;
    int freeSpace;
    freeSpace = width - 1 - 5 * ax.tickWidth - 4 * TICK_SPACE;
    result = floor(freeSpace / (4 * (ax.tickWidth + TICK_SPACE)));
    if (result < 1) return 1;
    return result;
}

int BarGraph::getOffset(int width, Axis ax)
{
    return int(ax.tickWidth / 2 + floor((width - getAxisWidth(getTickStep(width, ax), ax)) / 2));
}

int BarGraph::getAxisWidth(int ticks, Axis ax) const
{
    return ((5 + 4 * ticks) * ax.tickWidth + (4 + 4 * ticks) * TICK_SPACE);
}

void BarGraph::setBarColor(QColor color)
{
    this->axis.tickColor = color;
    drawAxis();
}

void BarGraph::setBackgroundColor(QColor color)
{
    this->axis.barColor = color;
    drawBar();
}

void BarGraph::setRange(double min, double max)
{
    if (min <= max) {
        this->bar.minimum = min;
        this->bar.maximum = max;
    } else {
        this->bar.minimum = max;
        this->bar.maximum = min;
    }
    setValue(this->bar.value);
}

void BarGraph::setValue(double value)
{
    if (value < this->bar.minimum) value = this->bar.minimum;
    else if (value > this->bar.maximum) value = this->bar.maximum;
    this->bar.value = value;
    drawBar(this->width(), this->axis, this->bar);
}

void BarGraph::drawBar(int width, Axis ax, Bar bar)
{
    int barWidth;
    barWidth = getBarWidth(ax);
    labelBarL->setStyleSheet(QString("QLabel {background-color: " + addTransparency(ax.barColor.name()) + ";}"));
    labelBarR->setStyleSheet(QString("QLabel {background-color: " + addTransparency(ax.barColor.name()) + ";}"));
    double barLength;
    barLength = getAxisWidth(getTickStep(width, ax), ax) - ax.tickWidth;
    int barStart;
    if (bar.minimum >= 0) {
        //single side
        labelBarL->setVisible(false);
        barStart = getOffset(width, ax) + int((ax.tickWidth + 1) / 2);
        barLength = ((bar.value - bar.minimum) * barLength) / (bar.maximum - bar.minimum);
        labelBarR->setGeometry(barStart + int(barLength), 0, width, barWidth);
    } else {
        //double side
        barStart = getOffset(width, ax) + int(((getAxisWidth(getTickStep(width, ax), ax)) + 1) / 2);
        barLength = barLength / 2;
        if (bar.value >= 0) {
            barLength = (bar.value * barLength) / bar.maximum;
            labelBarR->setGeometry(barStart + int(barLength), 0, width, barWidth);
            labelBarL->setGeometry(0, 0, barStart - ax.tickWidth, barWidth);
        } else {
            barLength = (bar.value * barLength) / bar.minimum;
            labelBarR->setGeometry(barStart, 0, width, barWidth);
            labelBarL->setGeometry(0, 0, barStart - ax.tickWidth - int(barLength), barWidth);
        }
        labelBarL->setVisible(true);
    }
}

void BarGraph::drawBar()
{
    drawBar(this->width(), this->axis, this->bar);
}

void BarGraph::setAlarmVisible(bool visible)
{
    this->alarm.visible = visible;
    // TODO
}

void BarGraph::setAlarmColors(QColor warning, QColor critical)
{
    this->alarm.critical = critical;
    this->alarm.warning = warning;
}

void BarGraph::setAlarm(double bCritical, double bWarning, double tWarining, double tCritical)
{
    this->alarm.topCritical = tCritical;
    this->alarm.topWarning = tWarining;
    this->alarm.bottomWarning = bWarning;
    this->alarm.bottomCritical = bCritical;
}

