/*
 * Part of Xlab Interface
 *
 * Created: 2018-10-26
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#ifndef MULTIMETERDISPLAY_H
#define MULTIMETERDISPLAY_H

#include <QWidget>
#include <QLayout>
#include <QToolButton>
#include <QPushButton>
#include <QMap>
#include <QLineEdit>
#include <QSizePolicy>
#include "bargraph.h"
#include "clickablelabel.h"
#include "multimeterdisplaybuttons.h"

class ModalSetValue;
class MultimeterDisplayOptions;

namespace Ui {
    class MultimeterDisplay;
}

class MultimeterDisplay : public QWidget
{
    Q_OBJECT

public:
    explicit MultimeterDisplay(QWidget *parent = nullptr);
    ~MultimeterDisplay();

    void connectOptions(MultimeterDisplayOptions *options, const QObject *object, const char *slot, const char *signal);
    void setUnit(QString unit);
    void setLabel(QString label);
    void setValue(double value);
    void setInfinity(void);
    void setNaN(void);
    void setValueFontSize(int size);
    void setValueFontFamily(QString family);
    void setLabelFontSize(int size);
    void setStretch(int label=30, int value=50, int unit=15, int accuracy=10);
    void setPrecision(int precision);
    void setUnitPower(int power);
    void setRange(double min, double max);
    void setRangeExceeded(int state);
    void enableCheckRange(bool enable);
    void setAccuracy(double unit, double percent);
    void setAccuracy(double unit, QString percent);
    void setAccuracyVisible(bool visible);
    void setHold(bool hold);
    void enableOffset(bool enable, double base);
    void enableExtreme(bool enable);
    void setExtreme(double min, double max);
    void clearExtreme(void);
    void setAlarm(double bCritical, double bWarning, double tWarining, double tCritical);
    void setAlarmLevel(int level);
    int readAlarm(void);
    void enableAlarm(bool enable);
    void enableExtendedPrecision(bool extended);
    void setButtonsVisible(bool visible);
    void enableButtons(bool enable);
    void enableBargraph(bool enable);
    void enableContexMenu(bool enable);

public slots:
    void applyConfiguration(void);
    void editOffsetBase(void);

protected:
    virtual void paintEvent(QPaintEvent *);
    virtual void resizeEvent(QResizeEvent *);

private:
    Ui::MultimeterDisplay *ui;
    MultimeterDisplayOptions *configuration;
    QAction *actionSettings;

    const int PRECISION_MAX = 24;
    const int POWER_MAX = 24;
    const QString INFINITY_STRING = "±\u221E  ";
    const QString NAN_STRING = "NaN  ";
    const QString TRANSPARENCY = "60";
    enum RangeState {
        RANGE_STATE_OverRange  = 1,
        RANGE_STATE_InRange    = 0,
        RANGE_STATE_BelowRange = -1
    };
    enum AlarmState {
        ALARM_STATE_Disabled   = -1,
        ALARM_STATE_OK         = 0,
        ALARM_STATE_Warning    = 1,
        ALARM_STATE_Critical   = 2,
        ALARM_STATE_Unknown    = 3
    };
    QString colorName;
    QString backgroundColorName;
    QString warningColorName;
    QString criticalColorName;

    QPixmap pixelmapOverRange;
    QPixmap pixelmapInRange;
    QPixmap pixelmapBelowRange;
    QSizePolicy policy;
    QLineEdit *lineEditBase;
    QMap<int, QString> mapPower;
    QWidget *background;

    typedef struct {
        double value;
        int power;
        int precision;
        bool infinity;
        bool nan;
    } Measure;
    typedef struct {
        double min;
        double max;
        bool external;
        bool enable;
    } Extreme;
    typedef struct {
        double min;
        double max;
        bool check;
        int exceeded;
    } Range;
    typedef struct {
        int precision;
        bool enable;
    } Extend;
    typedef struct {
        double unit;
        double percent;
        bool visible;
    } Accuracy;
    typedef struct {
        double base;
        bool enabled;
    } Offset;
    typedef struct {
        int level;
        double topCritical;
        double topWarning;
        double bottomWarning;
        double bottomCritical;
        bool external;
        bool enabled;
    } Alarm;
    typedef struct {
        int label;
        int value;
        int unit;
        int accuracy;
    } Stretch;
    Measure measure;
    Extreme extreme;
    Range range;
    Extend extend;
    Accuracy accuracy;
    Offset offset;
    Alarm alarm;
    Stretch stretch;
    bool hold;
    QString unit;
    QString label;
    QFont valueFont;
    QFont labelFont;
    QFont extraFont;
    bool appendLabel;
    bool appendNewLine;
    bool buttonsVisible;
    bool buttonsEnabled;
    bool bargraphVisible;
    bool contexMenuEnable;
    bool isBackgroundColor;

    void preparePowers(void);
    void setBackgroundColor(QString color);
    void setColor(QString color);
    void refreshBackground(void);
    void showUnit(void);
    void showValue(void);
    void showAccuracy(void);
    void showExtreme(void);
    int checkPrecision(int precision);
    QString getCurrentUnit(void);
    QString valueToString(double value, int power, int precision);
    void loadRangePixmap(QSize size);
    void scaleLayout(void);
    void showRangeExceeded(int state);
    void showAlarm(int state);
    void addContexMenu(void);
    void scaleLayoutSpacing(void);
    QSize getLayoutSpace(QFont font);
    void setOffsetBase(double base);
    void applyPolicy(QSizePolicy policy);
    void setPolicyRetain(void);
    void setExtendedPrecision(int precision);
    void setValueFont(QString family, int size);
    void setLabelFont(QString family, int size);
    void refreshStretch(int label, int value, int unit, int accuracy);

private slots:
    void callSettings(void);
    void showContextMenu(const QPoint& pos);
    void copyAsText(void);
    void copyAsImage(void);
    void copyAsDouble(void);
    void onLineEditBaseEditingFinished(void);
    void onLineEditBaseReturnPressed(void);
    void toolButtonExtendChanged(bool checked);
    void toolButtonOffsetChanged(bool checked);
    void toolButtonHoldChanged(bool checked);
    void toolButtonClearChanged(void);

signals:
    void rangeExceeded(int status);
    void configurationRequest(QString component);

};



#endif // MULTIMETERDISPLAY_H
