/***************************************************************************
**                                                                        **
**  XprotolabInterface                                                    **
**  Copyright (C) 2013 Gabotronics                                        **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program.  If not, see http://www.gnu.org/licenses/.   **
**                                                                        **
****************************************************************************
**           Author: Syed Adnan Kamili                                    **
**          Contact: adnan.kamili@gmail.com                               **
**             Date: 13.10.13                                             **
**          Version: 0.1-1                                                **
****************************************************************************
****************************************************************************
**    Last modified:                                                      **
**                   2016-2017 Krzysztof Markiewicz                       **
**                   www.obbo.pl                                          **
**                   Qt Creator 4.4.1 Based on Qt 5.9.2                   **
****************************************************************************/

#ifndef XLABINTERFACE_H
#define XLABINTERFACE_H

#include <QMainWindow>
#include <QDesktopWidget>
#include <QTimer>
#include <QFileDialog>
#include <QMessageBox>
#include <QtEndian>
#include <QInputDialog>
#include <QSettings>
#include <QFile>
#include <QTextStream>
#include <QtSerialPort/QSerialPortInfo>
#include <QtSerialPort/QSerialPort>
#include "customtheme.h"
#include "qcustomplot.h"
#include "libusbdevice.h"
#include "oscyloskopx.h"
#include "qxmlputget.h"
#include "collapsiblepanel.h"
#include "about.h"
#include "pythonconsole.h"
#include "globaloptions.h"
#include "pythonconsoleoptions.h"
#include "scopeoptions.h"
#include "settings.h"
#include "options.h"
#include "multimeterdisplay.h"
#include "multimeterdisplayoptions.h"



enum MainTabs {
    MAIN_TAB_Osciloscope,
    MAIN_TAB_Mater,
    MAIN_TAB_Graph,
    MAIN_TAB_ProtocolSniffer,
    MAIN_TAB_Options
};

enum WindowFunction {
    FFT_FUNCTION_Rectangular,
    FFT_FUNCTION_Hamming,
    FFT_FUNCTION_Hann,
    FFT_FUNCTION_Blackman
};

enum Selected {
    isHCursorAHead,
    isHCursorBHead,
    isVCursorAHead,
    isVCursorBHead,
    isTriggerPixmap,
    isTriggerWin1Pixmap,
    isTriggerWin2Pixmap,
    isCH1Zero,
    isCH2Zero,
    isCHDZero,
    isNone
};

enum Trigger {
    TRIGGER_MODE_Rising,
    TRIGGER_MODE_Falling,
    TRIGGER_MODE_Dual,
    TRIGGER_MODE_Positive,
    TRIGGER_MODE_Negative,
    TRIGGER_MODE_Window
};

enum TriggerType{
    Window1,
    Window2,
    Other
};

// as ui->comboBoxTheme->currentIndex()
enum theme {
    DISPLAY_THEME_Dark,
    DISPLAY_THEME_Light,
    DISPLAY_THEME_Custom
};

enum main_logginglevel {
    LOGGING_LEVEL_Debug,
    LOGGING_LEVEL_Info,
    LOGGING_LEVEL_Warning,
    LOGGING_LEVEL_Critical,
    LOGGING_LEVEL_Fatal
};

#define TRIGGER_POST_MAX                32767
#define TRIGGER_POSX_SSR                24
#define CURSOR_DISTANCE_MAX             256      // usually = rangeMax / 2
#define TRIGGRT_WINDOW_DISTANCE_MIN     16

#define DATACACHE_SIZE                  16384

#define AWG_WAVE_TEMPLATE_LENGTH        512

#define TG                              9
#define GAIN_FACTOR                     0.78125       // = 2.5[V](ref) / 0.0125[V/pix] / 256(8bit)
#define AMPLITUDE_SLIDER_FACTOR         115.91
#define DUTY_SLIDER_FACTOR              (128 / 50.00064)
#define OFFSET_SLIDER_FACTOR            (32 / 0.50016)

#define BUTTON_STOP_SCOPE_TEXT_START    "START"
#define BUTTON_STOP_SCOPE_TEXT_STOP     "STOP"
#define BUTTON_STOP_SNIFFER_TEXT_START  "START"
#define BUTTON_STOP_SNIFFER_TEXT_STOP   "STOP"

#define METER_RESULT_REFRESH_Ms         400

typedef QVector<double> DataBuffer;

typedef struct {
    int posX;
    int posY;
    int posY1;
    int posY2;
    int offset;
    int bit;
    int size;
} ScopeTriggerCoord;

typedef struct {
    bool enable;
    bool visible;
    int mode;
    int source;
    int channel;
    ScopeTriggerCoord coord[3];
    bool ssRate;
    int posXmin;
    int posXmax;
    int posYmin;
    int posYmax;
    int posYWmin;
    int posYWmax;
    int posYDist;
} ScopeTrigger;

typedef struct {
    bool visible;
    int samplingrate;
    int AposX;
    int BposX;
    QColor color;\
    int posXmin;
    int posXmax;
} ScopeVCursor;

typedef struct {
    bool enabled;
    int headA;
    int headB;
} ScopeHCursorCoord;

typedef struct {
    bool visible;
    int channel;
    bool track;
    ScopeHCursorCoord coord[2];
    QColor color;
    int maxDistance;
} ScopeHCursor;

typedef struct {
    uint16_t first;
    uint16_t second;
    uint16_t last;
} mmzeros_t;


namespace Ui {
class XlabInterface;
}

class XlabInterface : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit XlabInterface(QWidget *parent = 0);
    ~XlabInterface();


private:
    void setupGrid(QCustomPlot*);
    void setupGraphs(QCustomPlot*);
    void setupTracers(QCustomPlot*);
    void setupCursors(QCustomPlot*);
    void setupItemLabels(QCustomPlot*);
    void setupScatterStyles(bool);
    void saveWavetoFile();
    QString saveWavetoString();
    void closeEvent(QCloseEvent*);
    void sendWaveType();
    void sweepCursorsUpdate();
    void updateSweepDuty(int);
    void parseCSV(QString, byte*);
    int sendCH1Controls(unsigned char *result, int length);
    void sendCH1Controls();
    int sendCH2Controls(unsigned char *result, int length);
    void sendCH2Controls();
    int sendCHDControls(unsigned char *result, int length);
    void sendCHDControls();
    void sendCHDBitControls();
    void sendTriggerControls();
    void sendMixedControls();
    void fftControlsPlotSet();
    void sendVerticalCursorA();
    void sendVerticalCursorB();
    void sendHorizontalCursorCH1A();
    void sendHorizontalCursorCH1B();
    void sendHorizontalCursorCH2A();
    void sendHorizontalCursorCH2B();
    void sendSweepTypeControls();
    void sendSnifferOptionControl();
    void sendSnifferUSARTControl();
    void displayControlsSet();
    void sniffControlsStartSet(QString command);
    void updateFrequency(double value);
    void updateSweepAmp(double);
    void setupValues();
    void fftWindowFunctionSet();
    void sendDecodeProtocol(int);
    void logToFile(QString, int level);
    void sendTriggerLevel(int);
    void sendTriggerWin1Level(int);
    void sendTriggerWin2Level(int);
    void sendTriggerPost(int value);
    void setupAppSetting();
    void sniffControlsEnable(bool);
    void triggerInit();
    void triggerSetEnable(bool enable);
    void triggerSetIcon();
    void triggerSetVisible(bool visible);
    void triggerSetMode(int mode);
    void triggerSetSlowSamplingRate(bool value);
    void triggerSetSource(int index);
    void triggerMoveTrigger(int curPosX, int curPosY, bool force = false);
    void triggerMoveWinTrigger(int, int, int, bool force = false);
    void triggerSetPosition(int posx, int posy);
    void triggerSetPositionCoord(double posx, double posy);
    void triggerSetPositionWin1(int posx, int posy);
    void triggerSetPositionWin1Coord(int posx, int posy);
    void triggerSetPositionWin2(int posx, int posy);
    void triggerSetPositionWin2Coord(int posx, int posy);
    void triggerSetRange(int xmin, int xmax, int ymax, int ywmax, int ydist);
    void triggerSetChannel(int channel);
    void triggerRefreshPos();
    void triggerSetChannelSize(int ch, int size);
    void triggerSetChannelBit(int ch, int bit);
    void triggerSetChannelOffset(int ch, int offset);
    double triggerPosYBit(int bit);
    int mapRange(int value, int oldMax, int oldMin, int newMax, int newMin);
    double mapRangeF(double value, double oldMax, double oldMin, double newMax, double newMin);
    int dataValueRangeTrunk(int value, int min, int max);
    int dataValueRangeTrunk64(qint64 value, qint64 min, qint64 max);
    double int16ToVolt_f(uint8_t gain, int16_t v0);
    double int8ToVolt_f(uint8_t gain, int8_t v0, int8_t v1);
    double int8ToVolt_f(uint8_t gain, int8_t v0);
    double samplex100ToFrequency_f(int32_t fpix, uint8_t samplingrate);
    QString removeTrailingZeros(QString str);
    void clearMMLabelsCH1AC();
    void clearMMLabelsCH2AC();
    int triggerPosXIntToPixel(int posX);
    int triggerPosYIntToPixel(int posY);
    int triggerPosYPixelToInt(int posYpx);
    QString pixelValueToVolt(double value, QString unit, int precision);
    double rawValueToVolt(double value, QString unit);
    void logicLabelPositionSet(int idx);
    int logicBitPos0(int m);
    QString getStringFromUnsignedChar( unsigned char *buffer, int length );
    void clearBuffer(unsigned char *buffer, int length);
    void setVerticalScrollBar(int lo, int up);
    void setHorizontalScrollBar(int lo, int up);
    void deviceSettingsExec();
    void zeroHeadRefresh();
    void vCursorRefresh();
    void vCursorRefreshA();
    void vCursorRefreshB();
    void vCursorInit();
    void vCursorSetRange(int min, int max);
    void vCursorSetVisible(bool enabled);
    void vCursorRefreshLabel();
    void vCursorSetPosition(int a, int b);
    void vCursorSetLabelVisible(bool enabled);
    void vCursorSetLabelColor(QColor color);
    double vCursorDeltaTime(int apos, int bpos, QString *unit, int div, int samplingrate);
    double vCursorFrequency(double deltatime, QString *unit, int samplingrate);
    double deltaToFrequency(double deltatime, int samplingrate);
    void deviceInfoRead(QString configFile);
    void scopeSetSizeX(int blocks);
    void changeWindowTitle(QString status);
    int mathRoundTo(double value, int grad);
    void sendMainModeControls(int mode);
    void sendMainConfiguration(byte config, int value);
    int sendMainConfigurationConfirm(unsigned char *result, int length, byte config, int value);
    void sendMainConfigurationFrequency(byte config, qint64 value);
    void sendChannel1GainControls(int value);
    void sendChannel2GainControls(int value);
    int sendDACControls(unsigned char *result, int length);
    void sendAutoTrigger(int value);
    void sendTriggerHold(int value);
    void sendSweepStart(int value);
    void sendSweepEnd(int value);
    void sendSweepSpeed(int value);
    void sendWaveConstantLevel(int value);
    void sendWaveAmplitude(int value);
    void sendWaveDuty(int value);
    void sendWaveOffset(int value);
    void sendSamplingRate(int value);
    void sendDACBufferBlocks(int value);
    void sendCHBufferBlocks(int value);
    void sendWaveFrequency(qint64 value);
    void sendSnifferStart();
    void sendSnifferStop();
    void sendSnifferClear();
    void sendCouterStart();
    void sendCouterStop();
    void sendScopeForce();
    void sniffProtocolSerialPrint(uint8_t *buffer, uint8_t *validbuffer, uint8_t validator, uint16_t idx, QPlainTextEdit *outwidget, bool firstloop);
    void sniffProtocolSPIPrint(uint8_t *buffer, uint16_t idx, QPlainTextEdit *outwidget, bool firstloop);
    void sniffProtocolI2CPrint(uint8_t *buffer, uint8_t *ackbuffer, uint16_t idx, QPlainTextEdit *outwidget);
    void sniffProtocolParallelPrint(uint8_t *buffer, uint16_t idx, QPlainTextEdit *outwidget, bool firstloop);
    void sendSnifferStrobeEdge(int value);
    void dataScopeHeaderGetData(frameconfig_t *head, uint8_t *data);
    void dataChannelMeterGetData(channela_t *meter, uint8_t *data);
    void dataChannelMeterGetSetitings(channela_t *meter, uint8_t *data);
    void dataSnifferHeaderGetData(sniffconfig_t *head, uint8_t *data);
    void sniffProtocolWidgetClear();
    void sniffStop();
    void scopeDataShow();
    void hCursorInit();
    void hCursorRefreshA();
    void hCursorRefreshB();
    void hCursorcSetPositionAuto(bool enabled, double minCh1, double maxCh1, double minCh2, double maxCh2);
    void hCursorRefreshLabel(bool enabled, int ch1gain, int ch2gain);
    void hCursorSetTracerVisible(bool enabled);
    void hCursorSetLabelVisible(bool enabled);
    void hCursorSetLabelColor(QColor color);
    void hCursorSetVisible(bool enabled);
    void hCursorSetChannelEnabled(int channel, bool enabled);
    void hCursorRefreshVisible();
    float frequencyRealValueCheck();
    void frequencyDigitsShowValue(double value);
    double frequencyDigitsToFrequency();
    double frequencySliderValueToInt(double value);
    void frequencyDigitIncrease(QLabel *digit, int range);
    void frequencyDigitDecrease(QLabel *digit, int range);
    void frequencyDigitsValueChanged();
    int frequencyIntToSliderValue(double value);
    void scopeSetTriggerModeButtonVisible(bool visible);
    void sendScopeStart();
    void sendScopeStop();
    void wavePlay(QString file);
    int deviceInitialize();
    void gridBoxSet(QColor color, Qt::PenStyle style);
    void counterButtonEnable(bool state);
    void channel1GainSliderChanged(int value, bool press);
    void channel2GainSliderChanged(int value, bool press);
    void channelPositionSliderChanged(int channel, int value, int *chzeropos, bool enabled);
    void channel1TraceChanged(bool checked);
    void channel2TraceChanged(bool checked);
    void channelLogicSizeSliderChanged();
    void channelLogicPositionSliderChanged(int value);
    void channelLogicTraceChanged();
    void displayIntensitySliderChanged(int value);
    void displayVectorsChanged(bool checked);
    void displayThemeChanged(int theme);
    void displayGridStyleChanged(int index);
    void cursorsVerticalChanged(bool checked);
    void cursorsHorizontalChanged(bool checked);
    void cursorsHorizontalTypeSet(int index);
    void cursorsSetChannelActive(int index);
    void scopeTriggerSourceChanged(int index);
    void scopeSetTriggerAutoTimer(double value);
    void scopeSetTriggerType();
    void scopeSetTriggerMode();
    void scopeRollModeChanged();
    void scopeElasticModeChanged();
    void scopeSamplingSliderChanged(int value);
    void meterFrequencyModeChanged();
    void meterTabChanged(int index);
    void awgDACBufferChanged(int index);
    void awgEnableChanged();
    void awgAmplitudeSliderChanged(int value);
    void awgDutyCycleSliderChanged(int value);
    void awgOffsetSliderChanged(int value);
    void sweepSpeedSliderChanged(int value);
    void sweepTypeChanged();
    void sweepStartFreqChanged(QSlider *start, QSlider *end);
    void sweepEndFreqChanged(QSlider *start, QSlider *end);
    void sniffProtocolTabChanged(int index);
    void scopeADCBufferChanged(int index);
    void mainTabChanged(int index);
    void horizontalScrollBarChanged(int position);
    void sniffPauseButtonChanged();
    void channelSetBoxTitle(QGroupBox *boxac, QGroupBox *boxdc, int ch, int gain, bool ac);
    void channel1ACDCChanged(bool press);
    void channel2ACDCChanged(bool press);
    void channel1DCButtonCheck(int gain, bool ac);
    void channel2DCButtonCheck(int gain, bool ac);
    int deviceProfileGet(QString version);
    void deviceSettingsSendInteractive();
    void bufferComboBoxItemChange(QComboBox *combo, int count);
    void bufferCheckMaxSize();
    byte scopeCH1ControlsValue();
    byte scopeCH2ControlsValue();
    byte scopeCHDControlsValue();
    uint32_t scopeGetMeasureFrequency(int8_t *buffer, const uint16_t length, int16_t min, int16_t max, mmzeros_t *cross);
    void scopeGetAverRMS(int8_t *buffer, mmzeros_t *cross, int16_t *aver, int16_t *rms);
    void scopeInitDatacache();
    void dataInitChannelMeterBuffer(channela_t *meter);
    void scopeRefreshSamplingRateLabel();
    void ch1vZeroRefresh();
    void closeApplication();
    void updateLabelPosition();

private slots:
    void on_playButton_clicked();
    void dataShow();
    void sniffDataShow();
    int deviceSettingsInitialize();
    void deviceSettingsSendBulk();
    void moveCursor(QMouseEvent*);
    void keyReleaseEvent(QKeyEvent*);
    void keyPressEvent(QKeyEvent*);
    void selectItem(QMouseEvent*);
    void deselectItem(QMouseEvent*);
    void saveAppSettings();
    void deviceSettingsSave();
    void deviceSettingsLoad();
    void itemDoubleClick(QCPAbstractItem*,QMouseEvent*);
    void setTheme(int, CustomColors *customColors = NULL);
    void on_connectButton_clicked();
    void xAxisChanged(QCPRange);
    void yAxisChanged(QCPRange);
    void on_stopButton_clicked();
    void on_radioButtonSquare_clicked();
    void on_radioButtonExpo_clicked();
    void on_radioButtonTriangle_clicked();
    void on_radioButtonSine_clicked();
    void on_radioButtonWhiteNoise_clicked();
    void on_radioButtonCustom_clicked();
    void on_samplingSlider_valueChanged(int value);
    void on_ch1PositionSlider_valueChanged(int value);
    void on_ch2PositionSlider_valueChanged(int value);
    void on_openCSVButton_clicked();
    void on_saveAWGButton_clicked();
    void on_checkBoxCH1Math_clicked();
    void on_checkBoxCH1Invert_clicked();
    void on_checkBoxCH1Trace_clicked();
    void on_checkBoxCH1Average_clicked();
    void on_checkBoxCH2Math_clicked();
    void on_checkBoxCH2Invert_clicked();
    void on_checkBoxCH2Trace_clicked();
    void on_checkBoxCH2Average_clicked();
    void on_radioButtonCH2Sub_clicked();
    void on_radioButtonCH1Sub_clicked();
    void on_checkBoxCHDTrace_clicked();
    void on_checkBoxCHDInvert_clicked();
    void on_checkBoxASCII_clicked();
    void on_checkBoxCHD0_clicked();
    void on_checkBoxCHD1_clicked();
    void on_checkBoxCHD2_clicked();
    void on_checkBoxCHD3_clicked();
    void on_checkBoxCHD4_clicked();
    void on_checkBoxCHD5_clicked();
    void on_checkBoxCHD6_clicked();
    void on_checkBoxCHD7_clicked();
    void on_radioButtonRisingEdge_clicked();
    void on_radioButtonFallingEdge_clicked();
    void on_radioButtonDual_clicked();
    void on_radioButtonPositiveSlope_clicked();
    void on_radioButtonNegativeSlope_clicked();
    void on_radioButtonWindow_clicked();
    void on_radioButtonFree_clicked();
    void on_radioButtonNormal_clicked();
    void on_radioButtonAuto_clicked();
    void on_radioButtonSingle_clicked();
    void on_checkBoxCircular_clicked();
    void on_rollMode_clicked();
    void on_checkBoxLogY_clicked();
    void on_checkBoxFFTTrace_clicked();
    void on_checkBoxIQFFT_clicked();
    void on_radioButtonRect_clicked();
    void on_radioButtonHamming_clicked();
    void on_radioButtonHann_clicked();
    void on_radioButtonBlackman_clicked();
    void on_xyMode_clicked();
    void on_radioButtonSniffNormal_clicked();
    void on_radioButtonSniffSingle_clicked();
    void on_comboBoxBaud_currentIndexChanged(int index);
    void on_comboBoxParity_currentIndexChanged(int index);
    void on_comboBoxStopBits_currentIndexChanged(int index);
    void on_comboBoxCPOL_currentIndexChanged(int index);
    void on_comboBoxCPHA_currentIndexChanged(int index);
    void on_forceButton_clicked();
    void on_ch1GainSlider_valueChanged(int value);
    void on_ch2GainSlider_valueChanged(int value);
    void on_horizontalScrollBar_valueChanged(int position);
    void on_doubleSpinBoxTrigHold_valueChanged(double arg1);
    void on_comboBoxTrigSource_currentIndexChanged(int index);
    void on_doubleSpinBoxTrigAuto_valueChanged(double arg1);
    void on_chdPositionSlider_valueChanged(int value);
    void on_sweepStartFreqSlider_valueChanged(int value);
    void on_sweepEndFreqSlider_valueChanged(int value);
    void on_sweepSpeedSlider_valueChanged(int value);
    void on_doubleSpinBoxAmp_valueChanged(double arg1);
    void on_amplitudeSlider_valueChanged(int value);
    void on_doubleSpinBoxDuty_valueChanged(double arg1);
    void on_dutyCycleSlider_valueChanged(int value);
    void on_doubleSpinBoxOffset_valueChanged(double arg1);
    void on_offsetSlider_valueChanged(int value);
    void on_frequencySlider_valueChanged(int value);
    void on_checkBoxPersistence_clicked();
    void on_checkBoxVectors_clicked();
    void on_elasticMode_clicked();
    void on_checkBoxCursorVertical_clicked();
    void on_protocolTabWidget_currentChanged(int index);
    void on_captureButton_clicked();
    void on_screenShotButton_clicked();
    void on_saveWave_clicked();
    void on_loadWave_clicked();
    void on_clearWaveButton_clicked();
    void on_intensitySlider_valueChanged(int value);
    void on_radioButtonCH1Multiply_clicked();
    void on_radioButtonCH2Multiply_clicked();
    void on_pauseSnifferButton_clicked();
    void on_recordButton_clicked();
    void updateCh1Label(int ch1ctrl, int ch2ctrl);
    void updateCh2Label(int ch2ctrl, int ch1ctrl);
    void setInfoText(QString text = "", bool mode = false);
    void disconnectDevice(bool mode = true);
    QString get4Digits(qreal number);
    QString get3Digits(qreal number);
    QString getDigitsPrecision(qreal number, int prec);
    void showCH1ZeroVerticalIndicator(bool show);
    void showCH2ZeroVerticalIndicator(bool show);
    void showCHDZeroVerticalIndicator(bool show);
    bool checkIfCorrectPort(QString name);
    void on_mainTabWidget_currentChanged(int index);
    void meterRequestData();
    void on_mmTabWidget_currentChanged(int index);
    void on_fcR10_clicked();
    void on_fcR7_clicked();
    void on_fcR6_clicked();
    void on_fcR5_clicked();
    void on_fcR4_clicked();
    void on_fcR3_clicked();
    void on_fcR2_clicked();
    void on_fcR1_clicked();
    void on_fcR0_clicked();
    void sendTriggerSource();
    void sendTriggerSource(int value);
    void sendGateSource();
    void clearMMLabelsFreq();
    void on_radioFreq_clicked();
    void on_radioPulse_clicked();
    void on_checkBoxCH1ACDC_clicked();
    void on_checkBoxCH2ACDC_clicked();
    void on_radioButtonSweepFrequency_clicked();
    void on_radioButtonSweepAmplitude_clicked();
    void on_radioButtonSweepDutyCycle_clicked();
    void on_radioButtonSweepOffset_clicked();
    void on_radioButtonSweepRising_clicked();
    void on_radioButtonSweepPingPong_clicked();
    void on_radioButtonSweepFalling_clicked();
    void on_radioButtonConstant_clicked();
    void on_pushButtonCalibrateADCOffset_clicked();
    void on_toolButtonDACOffsetPlus_clicked();
    void on_toolButtonDACOffsetMinus_clicked();
    void on_toolButtonDACOffsetOK_clicked();
    void on_toolButtonDACGainPlus_clicked();
    void on_toolButtonDACGainMinus_clicked();
    void on_toolButtonDACGainOK_clicked();
    void on_horizontalSliderConstant_valueChanged(int value);
    void on_spinBoxConstant_valueChanged(int arg1);
    void on_pushButtonCalibrateADCOffsetClear_clicked();
    void on_pushButtonCounterStart_clicked();
    void on_pushButtonCounteStop_clicked();
    void on_toolButtonCh1ADC24V_clicked();
    void on_toolButtonCh1ADC2V4_clicked();
    void on_toolButtonCh2ADC24V_clicked();
    void on_toolButtonCh2ADC2V4_clicked();
    void on_groupBoxCH2AC_clicked();
    void on_groupBoxCH1AC_clicked();
    void on_groupBoxCH1DC_clicked();
    void on_groupBoxCH2DC_clicked();
    void on_checkBoxScopeCursorsHorizontal_clicked();
    void logicLabelsRefresh();
    void on_chdSizeSlider_valueChanged();
    void on_verticalScrollBar_valueChanged(int value);
    void scopeStartForce();
    void on_radioButtonPinkNoise_clicked();
    void on_checkBoxAWBEnable_clicked();
    void on_comboBoxDACBuffer_currentIndexChanged(int index);
    void on_checkBoxmeterondev_clicked();
    void sendMeterFrequencyControl();
    void on_radioPeriod_clicked();
    void on_checkBoxFreqFastMode_clicked();
    void on_radioExtGate_clicked();
    void on_comboBoxADCBuffer_currentIndexChanged(int index);
    void on_radioFrequencyGatePeriod_clicked();
    void on_radioFrequencyGatePulse_clicked();
    void on_fcR10_2_clicked();
    void on_fcR7_2_clicked();
    void on_fcR6_2_clicked();
    void on_fcR5_2_clicked();
    void on_fcR4_2_clicked();
    void on_fcR3_2_clicked();
    void on_fcR2_2_clicked();
    void on_fcR1_2_clicked();
    void on_fcR0_2_clicked();
    void on_comboBoxUARTData_currentIndexChanged(int);
    void on_pushButtonSnifferClear_clicked();
    void on_checkBoxSynchro_clicked();
    void on_fcR11_clicked();
    void on_fcR11_2_clicked();
    void on_comboBoxSniffParallelStrobe_currentIndexChanged(int index);
    void on_checkBoxSniffParallelInputInvert_clicked();
    void on_toolButtonFrequencyDigit100000Up_clicked();
    void on_toolButtonFrequencyDigit100000Down_clicked();
    void on_toolButtonFrequencyDigit10000Up_clicked();
    void on_toolButtonFrequencyDigit10000Down_clicked();
    void on_toolButtonFrequencyDigit1000Up_clicked();
    void on_toolButtonFrequencyDigit1000Down_clicked();
    void on_toolButtonFrequencyDigit100Up_clicked();
    void on_toolButtonFrequencyDigit100Down_clicked();
    void on_toolButtonFrequencyDigit10Up_clicked();
    void on_toolButtonFrequencyDigit10Down_clicked();
    void on_toolButtonFrequencyDigit1Up_clicked();
    void on_toolButtonFrequencyDigit1Down_clicked();
    void on_toolButtonFrequencyDigit01Up_clicked();
    void on_toolButtonFrequencyDigit01Down_clicked();
    void on_toolButtonFrequencyDigit001Up_clicked();
    void on_toolButtonFrequencyDigit001Down_clicked();
    void on_startSnifferButton_clicked();
    void on_comboBoxSniffSSINV_currentIndexChanged(int);
    void on_toolButtonCH1GainPlus_clicked();
    void on_toolButtonCH1GainMinus_clicked();
    void on_toolButtonCH1GainOK_clicked();
    void on_toolButtonCH2GainPlus_clicked();
    void on_toolButtonCH2GainMinus_clicked();
    void on_toolButtonCH2GainOK_clicked();
    void on_checkBoxCH1Invert_toggled();
    void on_checkBoxCH2Invert_toggled();
    void on_collapsiblePanelAWGWaveform_collapsedChanged(bool collapsed);
    void on_collapsiblePanelAWGFrequency_collapsedChanged(bool collapsed);
    void on_collapsiblePanelAWGLevels_collapsedChanged(bool collapsed);
    void on_collapsiblePanelAWGSweep_checkedChanged(bool checked);
    void on_collapsiblePanelAWGSweep_collapsedChanged(bool collapsed);
    void on_actionExit_triggered();
    void on_actionAbout_triggered();
    void on_actionConsole_triggered();
    void on_collapsiblePanelScopeCursors_collapsedChanged(bool collapsed);
    void on_collapsiblePanelScopeDisplay_collapsedChanged(bool collapsed);
    void on_comboBoxScopeCursorsChannel_currentIndexChanged(int index);
    void on_comboBoxScopeCursorsHorizontalType_currentIndexChanged(int index);
    void on_collapsiblePanelScopeRecord_collapsedChanged(bool collapsed);
    void on_actionSettings_triggered();
    void callSettings(QString item);
    void applyModuleConfiguration();
    void applyScopeConfiguration();

public:
    int bufferSize;
    float windowsScaling;
    About *about = nullptr;

    void clickRadioButtonExpo();
    void clickRadioButtonTriangle();
    void clickRadioButtonSine();
    void clickRadioButtonWhiteNoise();
    void clickRadioButtonCustom();
    void clickRadioButtonConstant();
    void clickRadioButtonSquare();
    void clickRadioButtonPinkNoise();
    void setAwgFrequency(double f);
    void setAwgLevelsAmplitude(double val);
    void setAwgLevelsDutyCycle(double val);
    void setAwgLevelsOffset(double val);
    void setAwgSweepEnabled(bool enabled);
    void clickRadioButtonSweepFrequency();
    void clickRadioButtonSweepAmplitude();
    void clickRadioButtonSweepDutyCycle();
    void clickRadioButtonSweepOffset();
    void clickRadioButtonSweepDirectionRising();
    void clickRadioButtonSweepDirectionPingPong();
    void clickRadioButtonSweepDirectionFaling();
    void setAwgSweepSpeed(int val);
    void setAwgSweepStart(double val);
    void setAwgSweepEnd(double val);

signals:
    void applyConfigurationRequest();

protected:
    virtual void resizeEvent(QResizeEvent *);

private:
    Ui::XlabInterface *ui;
    int m_mainTimerDelay;
    QTimer dataTimer;
    LibUsbDevice usbDevice;
    bool isScrolling;
    QString filePath;
    static const int rangeMax = 512;    // 2*maxymalna amplituda sygnalu
    int hCursorAPos, hCursorBPos, vCursorAPos, vCursorBPos;
    ScopeVCursor vCursor;
    double *fftWindow;
    int ch1ZeroPos, ch2ZeroPos, chdZeroPos, hCursorAPosCh1, hCursorBPosCh1, hCursorAPosCh2, hCursorBPosCh2;
    QStringList rateText, gainText, gainSliderText;
    int freqValue[23], xmax, mode;
    bool bitChecked[8], itemIsSelected, captureRef, saveWave, displayLoadedWave;
    QCPItemTracer *phaseTracerAA, *phaseTracerAB, *phaseTracerBA, *phaseTracerBB;
    QCPItemStraightLine *hCursorA, *hCursorB, *vCursorA, *vCursorB, *ch1Zero, *ch1vZero, *ch2Zero, *chDZero;
    QCPItemPixmap *hCursorAHead, *hCursorBHead, *ch1ZeroHead, *ch1vZeroHead, *ch2ZeroHead, *chDZeroHead;
    QCPItemPixmap *triggerPixmap, *vCursorAHead, *vCursorBHead, *triggerWin1Pixmap, *triggerWin2Pixmap;
    QCPItemText *textLabelBit[8], *textLabelDeltaTime, *textLabelDeltaVoltage;
    QCPItemText *textLabelVoltageA, *textLabelVoltageB, *textLabelFrequency;
    QString sniffBuffer;
    int currentSelected;
    QVector<double> ch1RefBuffer, ch2RefBuffer, ch1SaveBuffer, ch2SaveBuffer;
    QList<DataBuffer> ch1PBuffer,ch2PBuffer;
    QVector<double> bitRef[8], bitSaveBuffer[8];
    bool initializing;
    QString defaultDir;
    QStringList triggerIconPathsG, triggerIconPathsR;
    CustomTheme customThemeDialog;

    /************** Graphs **************/
    QCPGraph *ch1BarGraph, *ch2BarGraph;
    QCPGraph *ch1Graph, *ch2Graph, *ch1RefGraph, *ch2RefGraph, *ch1PGraphs[TG], *ch2PGraphs[TG];
    QCPGraph *chdGraph[8], *chd0Graph[8], *chdRefGraph[8];

    /************** Graph Pens **********/
    QPen  ch1BarPen, ch2BarPen, ch1PPen[TG], ch2PPen[TG];
    QPen  ch1Pen, ch2Pen, ch1RefPen, ch2RefPen;
    QPen  chdPen[8], chdRefPen[8];

    /************** Grid Pens **********/
    QColor gridPenColor;
    Qt::PenStyle gridPenStyle;
    QPen  gridPen, axesPen;
    QBrush  backgroundBrush;

    bool logging;
    QTextStream *xm;
    int loggingCounter;
    QString logsToSave;
    int lastFrameScope;
    int lastFrameSniff;
    int sniffbufferfulfill;
    int sniffbufferfulfillrx;
    int sniffbufferfulfilltx;
    int sniffbufferfirstrxloop;
    int sniffbufferfirsttxloop;

    bool recordingWaves;
    QFile *recordWaveFile;
    QTextStream *recordWaveTextStream;
    QString dataToSave;
    QList<QVector<double> > loadedWave;
    QList<int> loadedFrequency;
    bool loadRecordedWave;
    int freqBeforeLoaded;
    bool m_repaint;
    int m_prevTabIndex;
    QTimer m_mmTimer;
    int xAxisMax, xAxisMin, yAxisMax, yAxisMin;
    int gridSize;
    int logicLabelPosition;
    int vCurBPos;
    byte awgBuffer[2 * AWG_WAVE_TEMPLATE_LENGTH];
    bool writeBulkConfig;
    bool awg_wave_update_disable;
    ScopeTrigger trigger;
    ScopeHCursor hCursor;
    uint8_t datacache[DATACACHE_SIZE];
    PythonConsole *pyconsole = nullptr;
    bool pythonConsoleIsRunning = false;
    Settings *settingsdialog = nullptr;
    // <--------------- build main configuration
    QMap<QString, QVariant *> *map_Option;
    GlobalOptions *tomap_GlobalOptions;
    QVariant *v_GlobalOptions;
    PythonConsoleOptions *tomap_PythonConsoleOptions;
    QVariant *v_PythonConsoleOptions;
    ScopeOptions *tomap_ScopeOptions;
    QVariant *v_ScopeOptions;
    MultimeterDisplayOptions *tomap_MultimeterDisplayOptions;
    QVariant *v_MultimeterDisplayOptions;
    //<---------------
};

#endif // XLABINTERFACE_H
